import os
from genesys.app.services import files_service, svn_service
from genesys.app.config import SVN_PARENT_PATH, SVN_PARENT_URL, PROJECTS_FOLDER, TEMPLATE_FILES_DIR
from configparser import NoOptionError
from genesys.app.utils import config_helpers

def create_task_file(blend_file_url, file_map_parser, task_type):
    file_folder_url = os.path.dirname(blend_file_url)
    try:
        task_type_map = file_map_parser.get('file_map', task_type).lower()
        if task_type_map == 'base':
            if not svn_service.is_svn_url(file_folder_url):
                svn_service.svn_make_dirs(file_folder_url, log_message=f'created {file_folder_url}')
            if not svn_service.is_svn_url(f'{blend_file_url}.blend'):
                svn_service.svn_import(path=os.path.join(TEMPLATE_FILES_DIR,'blender.blend'),
                                        repo_url=f'{blend_file_url}.blend',
                                        log_message=f'create {blend_file_url}.blend')
        elif task_type_map == 'none':
            pass
        else:
            if not svn_service.is_svn_url(file_folder_url):
                svn_service.svn_make_dirs(file_folder_url, log_message=f'created {file_folder_url}')
            if not svn_service.is_svn_url(f'{blend_file_url}_{task_type_map}.blend'):
                svn_service.svn_import(path=os.path.join(TEMPLATE_FILES_DIR,'blender.blend'),
                                        repo_url=f'{blend_file_url}_{task_type_map}.blend',
                                        log_message=f'create {blend_file_url}.blend')
    except NoOptionError:
        if not svn_service.is_svn_url(file_folder_url):
            svn_service.svn_make_dirs(file_folder_url, log_message=f'created {file_folder_url}')
        if not svn_service.is_svn_url(f'{blend_file_url}_{task_type}.blend'):
            svn_service.svn_import(path=os.path.join(TEMPLATE_FILES_DIR,'blender.blend'),
                                        repo_url=f'{blend_file_url}_{task_type}.blend',
                                        log_message=f'create {blend_file_url}.blend')
        file_map_parser.set('file_map', task_type, task_type)

def create_new_task_acl(all_users, base_svn_directory, acl_parser, file_map_parser, task_type):
    try:
        task_type_map = file_map_parser.get('file_map', task_type).lower()
        if task_type_map == 'base':
            svn_directory = f'{base_svn_directory}.blend'
            if svn_directory in acl_parser:
                pass
            else:
                for user in all_users:
                    if svn_directory in acl_parser:
                        acl_parser.set(svn_directory, user, '')
                    else:
                        acl_parser[svn_directory] = {
                            '@admin':'rw',
                            user:''
                        }
        elif task_type_map == 'none':
            pass
        else:
            svn_directory = f'{base_svn_directory}_{task_type_map}.blend'
            if svn_directory in acl_parser:
                pass
            else:
                for user in all_users:
                    if svn_directory in acl_parser:
                        acl_parser.set(svn_directory, user, '')
                    else:
                        acl_parser[svn_directory] = {
                            '@admin':'rw',
                            user:''
                        }
    except NoOptionError:
        svn_directory = f'{base_svn_directory}_{task_type}.blend'
        if svn_directory in acl_parser:
            pass
        else:
            for user in all_users:
                if svn_directory in acl_parser:
                    acl_parser.set(svn_directory, user, '')
                else:
                    acl_parser[svn_directory] = {
                        '@admin':'rw',
                        user:''
                    }

def delete_task_file(blend_file_url, file_map_parser, task_type):
    try:
        task_type_map = file_map_parser.get('file_map', task_type).lower()
        if task_type_map == 'base':
            if svn_service.is_svn_url(f'{blend_file_url}.blend'):
                svn_service.svn_delete(f'{blend_file_url}.blend', log_message=f'created {blend_file_url}.blend')
        elif task_type_map == 'none':
            pass
        else:
            if svn_service.is_svn_url(f'{blend_file_url}_{task_type_map}.blend'):
                svn_service.svn_delete(f'{blend_file_url}_{task_type_map}.blend', log_message=f'created {blend_file_url}_{task_type_map}.blend')
    except NoOptionError:
        if svn_service.is_svn_url(f'{blend_file_url}_{task_type}.blend'):
            svn_service.svn_delete(f'{blend_file_url}_{task_type}.blend', log_message=f'created {blend_file_url}_{task_type}.blend')
        # file_map_parser.set('file_map', task_type, task_type)


from flask import Blueprint, jsonify, request
from flask_restful import Api, Resource
from genesys.app.config import SVN_PARENT_PATH, SVN_PARENT_URL, PROJECTS_FOLDER, FILE_MAP
from genesys.app.blueprints.project.utils import create_project_folders, create_project_file_map
from genesys.app.services import svn_service, project_service, files_service
import os
import shutil
# from rq import Queue
# import redis

# r = redis.Redis()
# q = Queue(connection=r)

project = Blueprint('project', __name__)
api = Api(project)

class Project(Resource):
    """
    Create, rename, and delete project folder and repositories
    
    """
    def post(self, project_name):
        project_name = files_service.format_file_name(project_name)
        self.create_project(project_name)
        project_repo_url = os.path.join(SVN_PARENT_URL, project_name)
        return jsonify(message=f'project created', project_name=project_name, svn_url=project_repo_url)

    def put(self, project_name):
        data = request.get_json()
        old_project_name = files_service.format_file_name(data['old_project_name'])
        new_project_name = files_service.format_file_name(data['new_project_name'])
        project_repo_url = os.path.join(SVN_PARENT_URL, project_name)
        self.rename_project(old_project_name, new_project_name)
        return jsonify(message=f'project created', project_name=project_name, svn_url=project_repo_url)

    def delete(self, project_name):
        project_name = files_service.format_file_name(project_name)
        self.archive_project(project_name)
        return jsonify(message=f'project archived', project_name=project_name)

    def create_project(self, project_name):
        project_svn_folder = os.path.join(SVN_PARENT_PATH, project_name)
        project_repo_url = os.path.join(SVN_PARENT_URL, project_name)
        project_folder = os.path.join(PROJECTS_FOLDER, project_name)
        # svn_authz_path = os.path.join(SVN_PARENT_PATH, project_name, 'conf/authz')

        svn_service.create_svn_repo(project_svn_folder)
        #TODO remove checkout
        create_project_folders(project_repo_url)
        create_project_file_map(project_repo_url, file_map=FILE_MAP)
        # svn_service.svn_check_out(project_repo_url, project_folder)
        # svn_service.svn_commit_all(project_folder, 'event commit')
        svn_service.default_acl(project_name)

    def rename_project(self, old_project_name, new_project_name):
        """
        rename project folder and repository and also relocates svn repo
        """
        new_project_folder = os.path.join(PROJECTS_FOLDER, new_project_name)
        old_project_folder = os.path.join(PROJECTS_FOLDER, old_project_name)
        new_project_svn_folder = os.path.join(SVN_PARENT_PATH, new_project_name)
        old_project_svn_folder = os.path.join(SVN_PARENT_PATH, old_project_name)
        new_project_repo_url = os.path.join(SVN_PARENT_URL, new_project_name)
        # old_project_repo_url = os.path.join(svn_parent_url, old_project_name)
        shutil.move(old_project_svn_folder, new_project_svn_folder)
        shutil.move(old_project_folder, new_project_folder)
        svn_service.svn_relocate(new_project_repo_url, new_project_folder)

    def archive_project(self, project_name):
        project_svn_folder = os.path.join(SVN_PARENT_PATH, project_name)
        project_folder = os.path.join(PROJECTS_FOLDER, project_name)
        svn_archive_folder = os.path.join(SVN_PARENT_PATH, 'archive')

        if not os.path.exists(svn_archive_folder):
            os.makedirs(svn_archive_folder)

        shutil.make_archive(base_name=os.path.join(svn_archive_folder, project_name), format='zip',
                            root_dir= project_svn_folder)
        shutil.rmtree(project_svn_folder)
        # shutil.rmtree(project_folder)

api.add_resource(Project, '/project/<string:project_name>')




from flask import Blueprint, jsonify, request
from flask_restful import Api, Resource
from genesys.app.config import SVN_PARENT_PATH, SVN_PARENT_URL, PROJECTS_FOLDER, FILE_MAP
from genesys.app.blueprints.project.utils import create_project_folders, create_project_file_map
from genesys.app.services import svn_service, project_service, files_service
import os
import shutil
from genesys.app.utils import config_helpers
from genesys.app.blueprints.asset.utils import rename_asset_task_file, rename_task_acl_section
from configparser import ConfigParser
# from rq import Queue
# import redis

# r = redis.Redis()
# q = Queue(connection=r)

asset = Blueprint('asset', __name__)
api = Api(asset)

class Asset(Resource):
    def put(self, project_name):
        data = request.get_json()
        project_repo_url = os.path.join(SVN_PARENT_URL, project_name)
        self.rename_asset(data, project_name)
        return jsonify(message=f'project created', project_name=project_name, svn_url=project_repo_url)

    def rename_asset(self, data, project_name):
        for asset_task in data:
            file_map_parser = ConfigParser()
            acl_parser = ConfigParser()
            task_type = asset_task['task_type']
            project = asset_task['project']
            root = os.path.join(project['file_tree']['working']['mountpoint'], project['file_tree']['working']['root'], '')
            # replacing file tree mount point with genesys config mount point
            print(asset_task['working_file_path'])
            #TODO rename shot folder
            base_file_directory = asset_task['working_file_path'].split(root,1)[1]
            base_svn_directory = asset_task['base_svn_directory']

            projects_folder_mount_point = os.path.dirname(PROJECTS_FOLDER)

            blend_file_url = os.path.join(SVN_PARENT_URL, base_file_directory)
            if asset_task['entity_type'] == 'asset':
                new_blend_file_url = os.path.join(os.path.dirname(blend_file_url), asset_task['new_file_name'])
            elif asset_task['entity_type'] == 'shot':
                shot_folder = os.path.join(os.path.dirname(os.path.dirname(blend_file_url)), \
                    asset_task['new_file_name'].rsplit('_', 1)[1])
                new_blend_file_url = os.path.join(shot_folder, asset_task['new_file_name'])
                print('shot', new_blend_file_url)
                print(shot_folder)
            new_base_svn_directory = os.path.join(os.path.dirname(base_svn_directory), asset_task['new_file_name'])
            project_folder = os.path.join(PROJECTS_FOLDER, project_name)
            svn_authz_path = os.path.join(SVN_PARENT_PATH, project_name, 'conf/authz')
            file_map_url = os.path.join(SVN_PARENT_URL, project_name, '.conf/file_map')

            config_helpers.load_file_map(file_map_url, file_map_parser)
            config_helpers.load_config(svn_authz_path, acl_parser)
            print(blend_file_url)
            print(new_blend_file_url)
            rename_asset_task_file(
                old_blend_file_url=blend_file_url,
                new_blend_file_url=new_blend_file_url, 
                file_map_parser=file_map_parser, 
                task_type=task_type)
            rename_task_acl_section(
                old_base_svn_directory= base_svn_directory,
                base_svn_directory= new_base_svn_directory,
                file_map_parser= file_map_parser,
                acl_parser= acl_parser,
                task_type= task_type
                )
            config_helpers.write_file_map(file_map_url=file_map_url, file_map_parser=file_map_parser)
            config_helpers.write_config(svn_authz_path, acl_parser)

        # svn_commit_message = 'renamed asset'
        # svn_service.svn_commit_all(project_folder, svn_commit_message)


api.add_resource(Asset, '/asset/<string:project_name>')




import os
from configparser import ConfigParser
import shutil
from genesys.app.utils import config_helpers
from genesys.app.services import svn_service

def create_project_folders(project_repo):
    base_directories = ['.conf','edit', 'lib', 'refs', 'scenes', 'tools']
    lib_directories = ['chars', 'envs', 'maps', 'nodes', 'props']
    for directory in base_directories:
        url = os.path.join(project_repo, directory)
        svn_service.svn_mkdir(url=url, log_message=f'create {url}')
    for directory in lib_directories:
        url = os.path.join(project_repo, 'lib', directory)
        svn_service.svn_mkdir(url=url, log_message=f'create {url}')

def create_project_file_map(project_repo, file_map):
    file_map_parser = ConfigParser()
    file_map_url = os.path.join(project_repo, '.conf/file_map')
    file_map_parser['file_map'] = file_map
    config_helpers.write_file_map(file_map_url=file_map_url, file_map_parser=file_map_parser)


import os
from genesys.app.services import svn_service

def write_config(file_directory, config_parser):
    '''
        write data from a configuration parser
        to file
    '''
    with open(file_directory, 'w') as f:
        config_parser.write(f)

def load_config(file_directory, config_parser):
    '''
        load data from a configuration file 
        into a configuration parser instance
    '''
    with open(file_directory, 'r') as f:
        data = f.read()
    config_parser.read_string(data)

def write_file_map(file_map_url, file_map_parser):
    '''
        write file map from a configuration parser
        to file
    '''
    temp_dir = os.path.join(os.path.dirname(os.path.dirname(__file__)), '.temp')
    if not os.path.isdir(temp_dir):
        os.mkdir(temp_dir)
    file_name = os.path.basename(file_map_url)
    temp_file = os.path.join(temp_dir, file_name)
    with open(temp_file, 'w') as f:
        file_map_parser.write(f)
    if not svn_service.is_svn_url(file_map_url):
        svn_service.svn_import(path=temp_file, repo_url=file_map_url, log_message='add config')
    else:
        svn_service.svn_delete(url=file_map_url, log_message='deleted file_map_temporaly')
        svn_service.svn_import(path=temp_file, repo_url=file_map_url, log_message='add config')

def load_file_map(file_map_url, file_map_parser):
    '''
        load data from a configuration file 
        into a configuration parser instance
    '''
    file = svn_service.svn_read_file(file_map_url)
    file_map_parser.read_string(file)


import os
SVN_PARENT_PATH = os.getenv("SVN_PARENT_PATH", '/opt/svn/repositories')
SVN_PARENT_URL = os.getenv("SVN_PARENT_URL", "file:////opt/svn/repositories")
PROJECTS_FOLDER = os.getenv("PROJECTS_FOLDER", "/opt/projects")
FILE_MAP = {
            'shading':'base',
            'concept':'none',
            'modeling':'base',
            'rigging':'base',
            'storyboard':'none',
            'layout':'layout',
            'previz':'layout',
            'animation':'anim',
            'lighting':'lighting',
            'fx':'fx',
            'rendering':'lighting',
            'compositing':'comp',
        }
LOGIN_NAME = os.getenv("LOGIN_NAME", "email")
TEMPLATE_FILES_DIR =os.path.join(os.path.dirname(__file__), 'template_files')


import os
from configparser import ConfigParser, NoOptionError, NoSectionError
import shutil
from genesys.app.utils import config_helpers
from genesys.app.services import svn_service

def rename_asset_task_file(old_blend_file_url, new_blend_file_url, file_map_parser, task_type):
    try:
        print('renaming')
        task_type_map = file_map_parser.get('file_map', task_type).lower()
        if task_type_map == 'base':
            source = f'{old_blend_file_url}.blend'
            destination = f'{new_blend_file_url}.blend'
            if svn_service.is_svn_url(source):
                svn_service.svn_rename(source, destination, log_message='renamed')
        elif task_type_map == 'none':
            pass
        else:
            source = f'{old_blend_file_url}_{task_type_map}.blend'
            destination = f'{new_blend_file_url}_{task_type_map}.blend'
            if svn_service.is_svn_url(source):
                svn_service.svn_rename(source, destination, log_message='renamed')
    except NoOptionError:
        source = f'{old_blend_file_url}_{task_type}.blend'
        destination = f'{new_blend_file_url}_{task_type}.blend'
        if svn_service.is_svn_url(source):
            svn_service.svn_rename(source, destination, log_message='renamed')

def rename_section(parser, section_from, section_to):
    items = parser.items(section_from)
    parser.add_section(section_to)
    for item in items:
        parser.set(section_to, item[0], item[1])
    parser.remove_section(section_from)

def rename_task_acl_section(old_base_svn_directory, base_svn_directory, file_map_parser, acl_parser, task_type):
    try:
        task_type_map = file_map_parser.get('file_map', task_type).lower()
        if task_type_map == 'base':
            source = f'{old_base_svn_directory}.blend'
            destination = f'{base_svn_directory}.blend'
            rename_section(acl_parser, source, destination)
        elif task_type_map == 'none':
            pass
        else:
            source = f'{old_base_svn_directory}_{task_type_map}.blend'
            destination = f'{base_svn_directory}_{task_type_map}.blend'
            rename_section(acl_parser, source, destination)
    except NoOptionError:
        source = f'{old_base_svn_directory}_{task_type}.blend'
        destination = f'{base_svn_directory}_{task_type}.blend'
        rename_section(acl_parser, source, destination)
    except NoSectionError:
        # FIXME check if renamed version of file exist in acl
        # TODO skip task with same files
        pass


from flask import Blueprint, jsonify, request
from flask_restful import Api, Resource
from genesys.app.config import (SVN_PARENT_PATH,
                                SVN_PARENT_URL,
                                PROJECTS_FOLDER,
                                FILE_MAP,
                                LOGIN_NAME)
from genesys.app.blueprints.task.utils import create_task_file, create_new_task_acl, delete_task_file
from genesys.app.services import svn_service, project_service
from genesys.app.utils import config_helpers
import os
import shutil
from configparser import ConfigParser

task = Blueprint('task', __name__)
api = Api(task)

class Task(Resource):
    def post(self, project_name):
        data = request.get_json()
        project_repo_url = os.path.join(SVN_PARENT_URL, project_name)
        project = (data['project'])
        root = os.path.join(project['file_tree']['working']['mountpoint'], project['file_tree']['working']['root'],'')
        # replacing file tree mount point with genesys config mount point
        base_file_directory = data['base_file_directory'].split(root,1)[1]
        self.create_task_file(project_name=project_name,
                                base_file_directory=base_file_directory,
                                base_svn_directory=data['base_svn_directory'],
                                all_persons=data['all_persons'],
                                task_type=data['task_type'],
                                )
        return jsonify(message=f'project created')

    def delete(self, project_name):
        data = request.get_json()
        project_repo_url = os.path.join(SVN_PARENT_URL, project_name)
        project = (data['project'])
        root = os.path.join(project['file_tree']['working']['mountpoint'], project['file_tree']['working']['root'],'')
        # replacing file tree mount point with genesys config mount point
        base_file_directory = data['base_file_directory'].split(root,1)[1]
        self.delete_task_file(project_name, base_file_directory, task_type=data['task_type'])

    def create_task_file(self, project_name, base_file_directory, base_svn_directory, all_persons, task_type):
        file_map_parser = ConfigParser()
        acl_parser = ConfigParser()
        svn_authz_path = os.path.join(SVN_PARENT_PATH, project_name, 'conf/authz')
        blend_file_url = os.path.join(SVN_PARENT_URL, base_file_directory)
        all_users = [person[LOGIN_NAME] for person in all_persons]
        project_folder = os.path.join(PROJECTS_FOLDER, project_name)
        file_map_url = os.path.join(SVN_PARENT_URL, project_name, '.conf/file_map')

        config_helpers.load_file_map(file_map_url, file_map_parser)
        config_helpers.load_config(svn_authz_path, acl_parser)

        create_task_file(blend_file_url=blend_file_url,
                        file_map_parser=file_map_parser,
                        task_type=task_type)
        
        create_new_task_acl(all_users=all_users,
                            base_svn_directory=base_svn_directory,
                            acl_parser=acl_parser,
                            file_map_parser=file_map_parser,
                            task_type=task_type)

        config_helpers.write_file_map(file_map_url, file_map_parser)
        config_helpers.write_config(svn_authz_path, acl_parser)
        
        # svn_commit_message = f'genesys added and commit {base_file_directory}'
        # svn_service.svn_commit_all(project_folder, svn_commit_message)

    def delete_task_file(self, project_name, base_file_directory, task_type):
        file_map_parser = ConfigParser()
        acl_parser = ConfigParser()
        svn_authz_path = os.path.join(SVN_PARENT_PATH, project_name, 'conf/authz')
        # if asset_task['entity_type'] == 'asset':
        #     new_blend_file_url = os.path.join(os.path.dirname(blend_file_url), asset_task['new_file_name'])
        # elif asset_task['entity_type'] == 'shot':
        #     shot_folder = os.path.join(os.path.dirname(os.path.dirname(blend_file_url)), \
        #         asset_task['new_file_name'].rsplit('_', 1)[1])
        #     new_blend_file_url = os.path.join(shot_folder, asset_task['new_file_name'])
        #     print('shot', new_blend_file_url)
        #     print(shot_folder)
        blend_file_url = os.path.join(SVN_PARENT_URL, base_file_directory)
        print(blend_file_url)
        file_map_url = os.path.join(SVN_PARENT_URL, project_name, '.conf/file_map')

        config_helpers.load_file_map(file_map_url, file_map_parser)
        # config_helpers.load_config(svn_authz_path, acl_parser)
        delete_task_file(blend_file_url=blend_file_url, file_map_parser=file_map_parser, task_type=task_type)

        # config_helpers.write_file_map(file_map_url, file_map_parser)
        # config_helpers.write_config(svn_authz_path, acl_parser)

class Task_File_Access_Control(Resource):
    def put(self, project_name):
        data = request.get_json()
        self.update_svn_acl(data, project_name)
        project_repo_url = os.path.join(SVN_PARENT_URL, project_name)
        return jsonify(message=f'project created', project_name=project_name, svn_url=project_repo_url)

    def update_svn_acl(self, data, project_name):
        file_map_parser = ConfigParser()
        acl_parser = ConfigParser()

        svn_authz_path = os.path.join(SVN_PARENT_PATH, project_name, 'conf/authz')
        project_folder = os.path.join(PROJECTS_FOLDER, project_name)
        file_map_dir = os.path.join(project_folder, '.conf/file_map')
        config_helpers.load_config(file_map_dir, file_map_parser)
        config_helpers.load_config(svn_authz_path, acl_parser)

        svn_service.svn_update_acl(
            base_svn_directory=data['base_svn_directory'],
            acl_parser=acl_parser,
            file_map_parser=file_map_parser,
            task_type=data['task_type'],
            person=data['person'][LOGIN_NAME],
            permission='rw'
        )

        config_helpers.write_config(file_map_dir, file_map_parser)
        config_helpers.write_config(svn_authz_path, acl_parser)


api.add_resource(Task, '/task/<string:project_name>')
api.add_resource(Task_File_Access_Control, '/task_acl/<string:project_name>')


from flask import Blueprint, jsonify, request
from flask_restful import Api, Resource
from genesys.app.config import (SVN_PARENT_PATH,
                                SVN_PARENT_URL,
                                PROJECTS_FOLDER,
                                FILE_MAP,
                                LOGIN_NAME)
from genesys.app.blueprints.task.utils import create_task_file, create_new_task_acl, delete_task_file
from genesys.app.services import svn_service, project_service
from genesys.app.utils import config_helpers
from configparser import ConfigParser
from genesys import __version__

index = Blueprint('index', __name__)
api = Api(index)

class Index(Resource):
    def get(self):
        return {"api": "eaxum_zou", "version": __version__}


api.add_resource(Index, '/')


from genesys.app.config import SVN_PARENT_PATH, SVN_PARENT_URL, PROJECTS_FOLDER, FILE_MAP
from genesys.app.blueprints.project.utils import create_project_folders, create_project_file_map
from genesys.app.services import svn_service, project_service
import os
import shutil

def create_project(project_name):
    import time
    time.sleep(10)
    project_svn_folder = os.path.join(SVN_PARENT_PATH, project_name)
    project_repo_url = os.path.join(SVN_PARENT_URL, project_name)
    project_folder = os.path.join(PROJECTS_FOLDER, project_name)
    svn_authz_path = os.path.join(SVN_PARENT_PATH, project_name, 'conf/authz')

    svn_service.create_svn_repo(project_svn_folder)
    svn_service.svn_check_out(project_repo_url, project_folder)
    create_project_folders(project_folder)
    create_project_file_map(project_folder, file_map=FILE_MAP)
    svn_service.svn_commit_all(project_folder, 'event commit')

def rename_project(old_project_name, new_project_name):
    new_project_folder = os.path.join(PROJECTS_FOLDER, new_project_name)
    old_project_folder = os.path.join(PROJECTS_FOLDER, old_project_name)
    new_project_svn_folder = os.path.join(SVN_PARENT_PATH, new_project_name)
    old_project_svn_folder = os.path.join(SVN_PARENT_PATH, old_project_name)
    new_project_repo_url = os.path.join(SVN_PARENT_URL, new_project_name)
    # old_project_repo_url = os.path.join(svn_parent_url, old_project_name)
    shutil.move(old_project_svn_folder, new_project_svn_folder)
    shutil.move(old_project_folder, new_project_folder)
    svn_service.svn_relocate(new_project_repo_url, new_project_folder)